Module: Bio::DB::SAM::Library

Defined in:
lib/bio/db/sam/library.rb

Class Method Summary collapse

Class Method Details

.filenameObject

Return the path with the file name of the library for the specific operating system



8
9
10
11
12
13
14
15
16
17
18
19
20
# File 'lib/bio/db/sam/library.rb', line 8

def filename
  #TODO refactor this piece of code in all the files
  lib_os = case RUBY_PLATFORM
  when /linux/
    'so.1'
  when /darwin/
    '1.dylib'
  when /windows/
    'dll'  
  end

  File.join(File.expand_path(File.dirname(__FILE__)),'external',"libbam.#{lib_os}")
end